import React, { useEffect, useState } from 'react';
import { Add, Delete, Settings } from '@mui/icons-material';
import { DatabaseInfo, useAppDispatch, useDatabaseAPI, useSchemaGraph, useSessionCache } from '@graphpolaris/shared/lib/data-access';
import { updateCurrentDatabase } from '@graphpolaris/shared/lib/data-access/store/sessionSlice';
import { SettingsForm } from './forms/settings';
import { NewDatabaseForm } from './forms/AddDatabase/newdatabase';
import { LoadingSpinner } from '@graphpolaris/shared/lib/components/LoadingSpinner';
import { addError } from '@graphpolaris/shared/lib/data-access/store/configSlice';
import { DropdownButton, DropdownContainer, DropdownItemContainer } from '@graphpolaris/shared/lib/components/dropdowns';
import { clearQB } from '@graphpolaris/shared/lib/data-access/store/querybuilderSlice';
import { clearSchema } from '@graphpolaris/shared/lib/data-access/store/schemaSlice';

export default function DatabaseSelector({}) {
  const dispatch = useAppDispatch();
  const api = useDatabaseAPI();
  const session = useSessionCache();
  const schemaGraph = useSchemaGraph();
  const dbSelectionMenuRef = React.useRef<HTMLDivElement>(null);
  const [hovered, setHovered] = useState<string | null>(null);
  const [connecting, setConnecting] = useState<boolean>(false);
  const [dbSelectionMenuOpen, setDbSelectionMenuOpen] = useState<boolean>(false);
  const [settingsMenuOpen, setSettingsMenuOpen] = useState<boolean>(false);
  const [selectedDatabase, setSelectedDatabase] = useState<DatabaseInfo | null>(null);
  const [addDatabaseFormOpen, setAddDatabaseFormOpen] = useState<boolean>(false);

  useEffect(() => {
    const handleClickOutside = ({ target }: MouseEvent) => {
      if (dbSelectionMenuRef.current && !dbSelectionMenuRef.current.contains(target as Node)) {
        setDbSelectionMenuOpen(false);
      }
    };
    document.addEventListener('click', handleClickOutside);
    return () => {
      document.removeEventListener('click', handleClickOutside);
    };
  }, []);

  useEffect(() => {
    setConnecting(false);
  }, [schemaGraph]);

  useEffect(() => {
    let timeoutId: ReturnType<typeof setTimeout>;
    if (connecting) {
      timeoutId = setTimeout(() => {
        dispatch(addError("Couldn't establish connection"));
        setConnecting(false);
        dispatch(updateCurrentDatabase(undefined));
        dispatch(clearQB());
        dispatch(clearSchema());
      }, 10000);
    }

    return () => {
      if (timeoutId) clearTimeout(timeoutId);
    };
  }, [connecting]);

  return (
    <>
      <SettingsForm
        open={settingsMenuOpen}
        database={selectedDatabase}
        onClose={() => {
          setSettingsMenuOpen(false);
        }}
      />
      <NewDatabaseForm
        open={addDatabaseFormOpen}
        onClose={() => {
          setAddDatabaseFormOpen(false);
        }}
      />
      <DropdownContainer ref={dbSelectionMenuRef} className="w-[20rem]">
        <DropdownButton
          title={
            <div className="flex items-center">
              {connecting ? (
                <>
                  <LoadingSpinner />
                  <p className="ml-2 truncate">Connecting to {session.currentDatabase}</p>
                </>
              ) : session.currentDatabase ? (
                <>
                  <div className="h-2 w-2 rounded-full bg-success-500" />
                  <p className="ml-2 truncate">Connected DB: {session.currentDatabase}</p>
                </>
              ) : session.databases === undefined ? (
                <>
                  <LoadingSpinner />
                  <p className="ml-2">Retrieving databases</p>
                </>
              ) : session.databases?.length === 0 ? (
                <>
                  <p className="ml-2">Add your first Database</p>
                </>
              ) : (
                <>
                  <div className="h-2 w-2 rounded-full bg-secondary-500" />
                  <p className="ml-2">Select a database</p>
                </>
              )}
            </div>
          }
          onClick={() => {
            if (session.databases?.length === 0) setAddDatabaseFormOpen(true);
            else setDbSelectionMenuOpen(!dbSelectionMenuOpen);
          }}
        />

        {dbSelectionMenuOpen && session.databases !== undefined && (
          <DropdownItemContainer align="top-10 w-full">
            <li
              className="flex items-center p-2 hover:bg-secondary-50 cursor-pointer"
              onClick={(e) => {
                e.preventDefault();
                setDbSelectionMenuOpen(false);
                setConnecting(false);
                setAddDatabaseFormOpen(true);
              }}
              title="Add new database"
            >
              {session.databases.length === 0 ? (
                <>
                  <Add />
                  <p className="ml-2">Add your first database</p>
                </>
              ) : (
                <>
                  <Add />
                  <p className="ml-2">Add database</p>
                </>
              )}
            </li>
            {session.databases.map((db) => (
              <li
                key={db.Name}
                className="flex justify-between items-center px-4 py-2 hover:bg-primary-100 gap-2 cursor-pointer"
                onClick={(e) => {
                  if (db.Name !== session.currentDatabase) {
                    e.preventDefault();
                    setDbSelectionMenuOpen(false);
                    setConnecting(true);
                    dispatch(updateCurrentDatabase(db.Name));
                    dispatch(clearQB());
                    dispatch(clearSchema());
                  } else {
                    setDbSelectionMenuOpen(false);
                  }
                }}
                onMouseEnter={() => setHovered(db.Name)}
                onMouseLeave={() => setHovered(null)}
                title={`Connect to ${db.Name}`}
              >
                <div className={`h-[8px] w-[8px] rounded-full shrink-0 ${db.status ? 'bg-success-500' : 'bg-danger-500'}`} />
                <div className="w-full shrink min-w-0 flex flex-col">
                  <p className="truncate w-full shrink-0 min-w-0">{db.Name}</p>
                  <p className="bg-light text-2xs text-secondary-500 truncate w-fit shrink-0 min-w-0 max-w-full h-full border border-secondary-200 rounded-sm p-0.5">
                    {db.Protocol}
                    {db.URL}
                  </p>
                </div>
                {hovered === db.Name && (
                  <div className="flex items-center ml-2">
                    <div
                      className="text-secondary-700 hover:text-secondary-400 transition-colors duration-300"
                      onClick={(e) => {
                        e.preventDefault();
                        e.stopPropagation();
                        setSettingsMenuOpen(true);
                        setSelectedDatabase(db);
                      }}
                    >
                      <Settings />
                    </div>
                    <div
                      className="text-secondary-700 hover:text-secondary-400 transition-colors duration-300"
                      onClick={(e) => {
                        e.preventDefault();
                        dispatch(updateCurrentDatabase(undefined));
                        dispatch(clearQB());
                        dispatch(clearSchema());
                        api.DeleteDatabase(db.Name);
                      }}
                      title="Delete database"
                    >
                      <Delete />
                    </div>
                  </div>
                )}
              </li>
            ))}
          </DropdownItemContainer>
        )}
      </DropdownContainer>
    </>
  );
}
